@@ -9,8 +9,8 @@ Michael Hucka <mhucka@caltech.edu> -- Caltech Library
 Copyright
 ---------
 
-Copyright (c) 2020 by the California Institute of Technology.  This code is
-open-source software released under a 3-clause BSD license.  Please see the
+Copyright (c) 2020-2021 by the California Institute of Technology.  This code
+is open-source software released under a 3-clause BSD license.  Please see the
 file "LICENSE" for more information.
 '''
 
@@ -35,16 +35,41 @@ from .base import UIBase
 # Constants.
 # .............................................................................
 
-_CLI_THEME = Theme({
-    'info'        : 'green3',
-    'warn'        : 'orange1',
-    'warning'     : 'orange1',
-    'alert'       : 'red',
-    'alert_fatal' : 'bold red',
-    'fatal'       : 'bold red',
-    'standout'    : 'bold green1',
-    'banner'      : 'green3',
-})
+# I haven't found a reasonable way to switch colors based on whether the
+# user's terminal background color is dark or light -- there seems to be no
+# universal way to get that information for every terminal emulator, due to
+# how they are implemented.  So, the following is an attempt to pick a single
+# set of colors that will work on both dark and bright color backgrounds.
+# The switch on Windows-versus-other is because when testing on Windows, I
+# get noticeably different color shades and brightness if I use cmd.exe vs
+# Cmder, and those are different *again* from my iTerm2 defaults on macOS.
+# So here I'm trying to find some compromise that will work in most cases.
+
+if sys.platform.startswith('win'):
+    # Note: Microsoft's Terminal (and I guess some others on Windows) can't show
+    # bold (2021-06-29). C.f. https://github.com/microsoft/terminal/issues/109
+    # The following style still uses bold in case that changes in the future.
+    _CLI_THEME = Theme({
+        'info'        : 'green3',
+        'warn'        : 'orange1',
+        'warning'     : 'orange1',
+        'alert'       : 'red',
+        'alert_fatal' : 'bold red',
+        'fatal'       : 'bold red',
+        'standout'    : 'bold dark_sea_green2',
+        'banner'      : 'green3',
+    })
+else:
+    _CLI_THEME = Theme({
+        'info'        : 'dark_sea_green4',
+        'warn'        : 'orange1',
+        'warning'     : 'orange1',
+        'alert'       : 'red',
+        'alert_fatal' : 'bold red',
+        'fatal'       : 'bold red',
+        'standout'    : 'bold chartreuse3',
+        'banner'      : 'dark_sea_green4',
+    })
 
 
 # Exported classes.
@@ -72,7 +97,8 @@ class CLI(UIBase):
             fancy_text = f'Welcome to [standout]{name}[/]: {subtitle}'
             text = fancy_text if use_color else plain_text
             terminal_width = shutil.get_terminal_size().columns or 80
-            padding = (terminal_width - len(plain_text) - 2) // 2
+            odd_adjustment = 0 if (terminal_width % 2 == 0) else 2
+            padding = (terminal_width - len(plain_text) - 2 - odd_adjustment) // 2
             # Queueing up this message now will make it the 1st thing printed.
             box_style = DOUBLE_EDGE if use_color else ASCII
             self._print_or_queue(Panel(text, style = 'banner', box = box_style,
@@ -140,7 +166,7 @@ class CLI(UIBase):
 
 
     def confirm(self, question):
-        '''Asks a yes/no question of the user, on the command line.'''
+        '''Ask a yes/no question of the user, on the command line.'''
         return input(f'{question} (y/n) ').startswith(('y', 'Y'))
 
 
@@ -150,7 +176,7 @@ class CLI(UIBase):
 
 
     def login_details(self, prompt, user = None, pswd = None):
-        '''Returns a tuple of user, password, and a Boolean indicating
+        '''Return a tuple of user, password, and a Boolean indicating
         whether the user cancelled the dialog.  If 'user' is provided, then
         this method offers that as a default for the user.  If both 'user'
         and 'pswd' are provided, both the user and password are offered as
@@ -173,6 +199,23 @@ class CLI(UIBase):
         except KeyboardInterrupt:
             return user, pswd, True
 
+
+    def validated_input(self, message, default_value, is_valid):
+        '''Get validated input from the user, optionally with a default value.'''
+        while True:
+            if __debug__: log(f'asking user: "{message} [{default_value}]"')
+            default = (' [' + default_value + ']') if default_value else ''
+            value = input(message + default + ': ')
+            if default_value and value == '':
+                if __debug__: log(f'user chose default value "{default_value}"')
+                return default_value
+            elif is_valid(value):
+                if __debug__: log(f'got "{value}" from user')
+                return value
+            else:
+                self.alert(f'"{value}" does not appear valid for {message}')
+                return None
+
 
 # Miscellaneous utilities
 # .............................................................................
